// Copyright 2022 Esri
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.


import UIKit
import ArcGIS

class ViewController: UIViewController {

    @IBOutlet var mapView: AGSMapView!
    private let map = AGSMap(basemapStyle: .arcGISTopographic)

    let featureTable = AGSServiceFeatureTable(url: URL(string: "https://services3.arcgis.com/GVgbJbqm8hXASVYi/arcgis/rest/services/LA_County_Parcels/FeatureServer/0")!)

    let optionsButton: UIButton = {
        let button = UIButton()
        button.backgroundColor = .lightGray
        button.setTitleColor(.blue, for: .normal)
        button.setTitle("Choose a SQL 'where' clause", for: .normal)
        button.translatesAutoresizingMaskIntoConstraints = false
        return button
    }()

    private func setupMap() {
        mapView.map = map
        mapView.setViewpoint(
            AGSViewpoint(
                latitude: 34.02700,
                longitude: -118.80500,
                scale: 72_000
            )
        )

        // Set selection color.
        mapView.selectionProperties.color = .yellow

    }

    private func setupLayer() {
        // Add feature layer to map's operational layers.
        let parcelsFeatureLayer = AGSFeatureLayer(featureTable: featureTable)
        mapView.map?.operationalLayers.add(parcelsFeatureLayer)
    }

    private func setupButton() {

        // Add button as subview to the view. 
        view.addSubview(optionsButton)

        // Add layout constraints defining the position of the button
        optionsButton.bottomAnchor.constraint(equalTo: view.bottomAnchor).isActive = true
        optionsButton.leftAnchor.constraint(equalTo: view.leftAnchor).isActive = true
        optionsButton.rightAnchor.constraint(equalTo: view.rightAnchor).isActive = true
        optionsButton.heightAnchor.constraint(equalToConstant: 60 + self.view.safeAreaInsets.bottom).isActive = true

        // Define method to call when button is tapped.
        optionsButton.addTarget(self, action: #selector(populateOptions), for: .touchUpInside)

    }

    @objc func populateOptions() {

        // Populate query options.
        let alert = UIAlertController(title: "Choose a SQL where clause.", message: nil, preferredStyle: .actionSheet)
        let whereClauses = ["UseType = 'Government'", "UseType = 'Residential'", "UseType = 'Irrigated Farm'", "TaxRateArea = 10853", "TaxRateArea = 10860", "Roll_LandValue > 1000000", "Roll_LandValue < 1000000"]
        whereClauses.forEach { whereClause in
            let action = UIAlertAction(title: whereClause, style: .default) { [unowned self] _ in
                executeQuery(whereClause: whereClause)
            }
            alert.addAction(action)
        }

        // Present query options.
        alert.popoverPresentationController?.sourceView = optionsButton
        alert.popoverPresentationController?.sourceRect = optionsButton.frame
        alert.preferredContentSize = CGSize(width: 300, height: 200)
        present(alert, animated: true)

    }

    func executeQuery(whereClause: String) {
        // Get the parcels layer from the map
        guard let parcelsFeatureLayer = map.operationalLayers.firstObject as? AGSFeatureLayer else {
            //... handle error
            return
        }
        
        // Clear any existing selection.
        parcelsFeatureLayer.clearSelection()
        
        // Create the query parameters using the where expression and extent passed in.
        let queryParams = AGSQueryParameters()
        queryParams.whereClause = whereClause
        queryParams.geometry = mapView.visibleArea?.extent
        queryParams.returnGeometry = true
        
        featureTable.queryFeatures(with: queryParams) { queryResult, error in
            if let _ = error {
                //... handle error
            } else if let features = queryResult?.featureEnumerator().allObjects as? [AGSArcGISFeature] {
                parcelsFeatureLayer.select(features)
            }
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        setupMap()

        setupLayer()

        setupButton()

    }

}



